import urllib
import urllib2

from django.core.management.base import CommandError
try:
    from django.utils import simplejson as json
except ImportError:
    import json
from django.conf import settings

def call(method, data, post=False):
    """
    Calls `method` from the DISQUS API with data either in POST or GET.
    Returns deserialized JSON response.
    """
    url = "%s%s" % ('http://disqus.com/api/', method)
    if post:
        # POST request
        url += "/"
        data = urllib.urlencode(data)
    else:
        # GET request
        url += "?%s" % urllib.urlencode(data)
        data = ''
    res = json.load(urllib2.urlopen(url, data))
    if not res['succeeded']:
        raise CommandError("'%s' failed: %s\nData: %s" % (method, res['code'], data))
    return res['message']



import base64
import hashlib
import hmac
import json as simplejson
import time

from django import template
from django.conf import settings
from django.contrib.sites.models import Site
from django.utils.functional import curry
from django.utils.encoding import force_unicode

register = template.Library()

# Set the disqus_developer variable to 0/1. Default is 0
@register.simple_tag(takes_context=True)
def set_disqus_developer(context, disqus_developer):
    context['disqus_developer'] = disqus_developer
    return ""

# Set the disqus_identifier variable to some unique value. Defaults to page's URL
@register.simple_tag(takes_context=True)
def set_disqus_identifier(context, *args):
    context['disqus_identifier'] = "".join(args)
    return ""

# Set the disqus_url variable to some value. Defaults to page's location
@register.simple_tag(takes_context=True)
def set_disqus_url(context, *args):
    context['disqus_url'] = "".join(args)
    return ""

# Set the disqus_title variable to some value. Defaults to page's title or URL
@register.simple_tag(takes_context=True)
def set_disqus_title(context, disqus_title):
    context['disqus_title'] = disqus_title
    return ""

# Set the disqus_category_id variable to some value. No default. See
# http://help.disqus.com/customer/portal/articles/472098-javascript-configuration-variables#disqus_category_id
@register.simple_tag(takes_context=True)
def set_disqus_category_id(context, disqus_category_id):
    context['disqus_category_id'] = disqus_category_id
    return ""

def get_config(context):
    """
    return the formatted javascript for any disqus config variables
    """
    conf_vars = ['disqus_developer', 'disqus_identifier', 'disqus_url',
        'disqus_title', 'disqus_category_id']

    output = []
    for item in conf_vars:
        if item in context:
            output.append('\tvar %s = "%s";' % (item, context[item]))
    return '\n'.join(output)

@register.simple_tag(takes_context=True)
def disqus_dev(context):
    """
    Return the HTML/js code to enable DISQUS comments on a local
    development server if settings.DEBUG is True.
    """
    if settings.DEBUG:
        return """<script type="text/javascript">
    var disqus_developer = 1;
    var disqus_url = '//%s%s';
</script>""" % (Site.objects.get_current().domain, context['request'].path)
    return ""

@register.simple_tag(takes_context=True)
def disqus_sso(context):
    """
    Return the HTML/js code to enable DISQUS SSO - so logged in users on
    your site can be logged in to disqus seemlessly.
    """
    # we have to make it str rather than unicode or the HMAC blows up
    DISQUS_SECRET_KEY = str(getattr(settings, 'DISQUS_SECRET_KEY', None))
    if DISQUS_SECRET_KEY is None:
        return "<p>You need to set DISQUS_SECRET_KEY before you can use SSO</p>"
    DISQUS_PUBLIC_KEY = getattr(settings, 'DISQUS_PUBLIC_KEY', None)
    if DISQUS_PUBLIC_KEY is None:
        return "<p>You need to set DISQUS_PUBLIC_KEY before you can use SSO</p>"
    user = context['user']
    if user.is_anonymous():
        return ""
    # create a JSON packet of our data attributes
    data = simplejson.dumps({
        'id': user.id,
        'username': user.username,
        'email': user.email,
    })
    # encode the data to base64
    message = base64.b64encode(data)
    # generate a timestamp for signing the message
    timestamp = int(time.time())
    # generate our hmac signature
    sig = hmac.HMAC(DISQUS_SECRET_KEY, '%s %s' % (message, timestamp), hashlib.sha1).hexdigest()

    # return a script tag to insert the sso message
    return """<script type="text/javascript">
var disqus_config = function() {
this.page.remote_auth_s3 = "%(message)s %(sig)s %(timestamp)s";
this.page.api_key = "%(pub_key)s";
}
</script>""" % dict(
        message=message,
        timestamp=timestamp,
        sig=sig,
        pub_key=DISQUS_PUBLIC_KEY,
    )

@register.inclusion_tag('disqus/num_replies.html', takes_context=True)
def disqus_num_replies(context, shortname=''):
    """
    Return the HTML/js code which transforms links that end with an
    #disqus_thread anchor into the threads comment count.
    """
    shortname = getattr(settings, 'DISQUS_WEBSITE_SHORTNAME', shortname)

    return {
        'shortname': shortname,
        'config': get_config(context),
    }

@register.inclusion_tag('disqus/recent_comments.html', takes_context=True)
def disqus_recent_comments(context, shortname='', num_items=5, excerpt_length=200, hide_avatars=0, avatar_size=32):
    """
    Return the HTML/js code which shows recent comments.

    """
    shortname = getattr(settings, 'DISQUS_WEBSITE_SHORTNAME', shortname)

    return {
        'shortname': shortname,
        'num_items': num_items,
        'hide_avatars': hide_avatars,
        'avatar_size': avatar_size,
        'excerpt_length': excerpt_length,
        'config': get_config(context),
    }

@register.inclusion_tag('disqus/show_comments.html', takes_context=True)
def disqus_show_comments(context, shortname=''):
    """
    Return the HTML code to display DISQUS comments.
    """
    shortname = getattr(settings, 'DISQUS_WEBSITE_SHORTNAME', shortname)
    return {
        'shortname': shortname,
        'config': get_config(context),
    }



from optparse import make_option
import os.path

from django.conf import settings
from django.contrib import comments
from django.contrib.sites.models import Site
from django.core.management.base import NoArgsCommand
try:
    from django.utils import simplejson as json
except ImportError:
    import json

from disqus.api import DisqusClient


class Command(NoArgsCommand):
    option_list = NoArgsCommand.option_list + (
        make_option('-d', '--dry-run', action="store_true", dest="dry_run",
                    help='Does not export any comments, but merely outputs' +
                         'the comments which would have been exported.'),
        make_option('-s', '--state-file', action="store", dest="state_file",
                    help="Saves the state of the export in the given file " +
                         "and auto-resumes from this file if possible."),
    )
    help = 'Export comments from contrib.comments to DISQUS'
    requires_model_validation = False

    def _get_comments_to_export(self, last_export_id=None):
        """Return comments which should be exported."""
        qs = comments.get_model().objects.order_by('pk')\
                .filter(is_public=True, is_removed=False)
        if last_export_id is not None:
            print "Resuming after comment %s" % str(last_export_id)
            qs = qs.filter(id__gt=last_export_id)
        return qs

    def _get_last_state(self, state_file):
        """Checks the given path for the last exported comment's id"""
        state = None
        fp = open(state_file)
        try:
            state = int(fp.read())
            print "Found previous state: %d" % (state,)
        finally:
            fp.close()
        return state

    def _save_state(self, state_file, last_pk):
        """Saves the last_pk into the given state_file"""
        fp = open(state_file, 'w+')
        try:
            fp.write(str(last_pk))
        finally:
            fp.close()

    def handle(self, **options):
        current_site = Site.objects.get_current()
        client = DisqusClient()
        verbosity = int(options.get('verbosity'))
        dry_run = bool(options.get('dry_run'))
        state_file = options.get('state_file')
        last_exported_id = None

        if state_file is not None and os.path.exists(state_file):
            last_exported_id = self._get_last_state(state_file)

        comments = self._get_comments_to_export(last_exported_id)
        comments_count = comments.count()
        if verbosity >= 1:
            print "Exporting %d comment(s)" % comments_count

        # if this is a dry run, we output the comments and exit
        if dry_run:
            print comments
            return
        # if no comments were found we also exit
        if not comments_count:
            return

        # Get a list of all forums for an API key. Each API key can have
        # multiple forums associated. This application only supports the one
        # set in the DISQUS_WEBSITE_SHORTNAME variable
        forum_list = client.get_forum_list(user_api_key=settings.DISQUS_API_KEY)
        try:
            forum = [f for f in forum_list\
                     if f['shortname'] == settings.DISQUS_WEBSITE_SHORTNAME][0]
        except IndexError:
            raise CommandError("Could not find forum. " +
                               "Please check your " +
                               "'DISQUS_WEBSITE_SHORTNAME' setting.")

        # Get the forum API key
        forum_api_key = client.get_forum_api_key(
            user_api_key=settings.DISQUS_API_KEY,
            forum_id=forum['id'])

        for comment in comments:
            if verbosity >= 1:
                print "Exporting comment '%s'" % comment

            # Try to find a thread with the comments URL.
            url = 'http://%s%s' % (
                current_site.domain,
                comment.content_object.get_absolute_url())
            thread = client.get_thread_by_url(
                url=url,
                forum_api_key=forum_api_key)

            # if no thread with the URL could be found, we create a new one.
            # to do this, we first need to create the thread and then
            # update the thread with a URL.
            if not thread:
                thread = client.thread_by_identifier(
                    forum_api_key=forum_api_key,
                    identifier=unicode(comment.content_object),
                    title=unicode(comment.content_object),
                )['thread']
                client.update_thread(
                    forum_api_key=forum_api_key,
                    thread_id=thread['id'],
                    url=url)

            # name and email are optional in contrib.comments but required
            # in DISQUS. If they are not set, dummy values will be used
            client.create_post(
                forum_api_key=forum_api_key,
                thread_id=thread['id'],
                message=comment.comment.encode('utf-8'),
                author_name=comment.userinfo.get('name',
                                                 'nobody').encode('utf-8'),
                author_email=comment.userinfo.get('email',
                                                  'nobody@example.org'),
                author_url=comment.userinfo.get('url', ''),
                created_at=comment.submit_date.strftime('%Y-%m-%dT%H:%M'))
            if state_file is not None:
                self._save_state(state_file, comment.pk)



from optparse import make_option

from django.core.management.base import NoArgsCommand, CommandError
try:
    from django.utils import simplejson as json
except ImportError:
    import json

from disqus.api import DisqusClient


class Command(NoArgsCommand):
    option_list = NoArgsCommand.option_list + (
        make_option('--indent', default=None, dest='indent', type='int',
            help='Specifies the indent level to use when pretty-printing output'),
        make_option('--filter', default='', dest='filter', type='str',
            help='Type of entries that should be returned'),
        make_option('--exclude', default='', dest='exclude', type='str',
            help='Type of entries that should be excluded from the response'),
    )
    help = 'Output DISQUS data in JSON format'
    requires_model_validation = False

    def handle(self, **options):
        from django.conf import settings

        client = DisqusClient()
        indent = options.get('indent')
        filter_ = options.get('filter')
        exclude = options.get('exclude')

        # Get a list of all forums for an API key. Each API key can have
        # multiple forums associated. This application only supports the one
        # set in the DISQUS_WEBSITE_SHORTNAME variable
        forum_list = client.get_forum_list(user_api_key=settings.DISQUS_API_KEY)
        try:
            forum = [f for f in forum_list\
                     if f['shortname'] == settings.DISQUS_WEBSITE_SHORTNAME][0]
        except IndexError:
            raise CommandError("Could not find forum. " +
                               "Please check your " +
                               "'DISQUS_WEBSITE_SHORTNAME' setting.")
        posts = []
        has_new_posts = True
        start = 0
        step = 100
        while has_new_posts:
            new_posts = client.get_forum_posts(
                user_api_key=settings.DISQUS_API_KEY,
                forum_id=forum['id'],
                start=start,
                limit=start+step,
                filter=filter_,
                exclude=exclude)
            if not new_posts:
                has_new_posts = False
            else:
                start += step
                posts.append(new_posts)
        print json.dumps(posts, indent=indent)



#!/usr/bin/env python

from setuptools import setup, find_packages

setup(
    name='django17-disqus',
    version='0.4.3',
    description='Export comments and integrate DISQUS into your Django website. Fix for Django 1.7. Thanks Arthur...',
    author='Alejandro Molina',
    author_email='jalemolina@gmail.com',
    url='https://github.com/jalemolina/django-disqus',
    license='New BSD License',
    classifiers=[
      'Framework :: Django',
      'Intended Audience :: Developers',
      'License :: OSI Approved :: BSD License',
      'Programming Language :: Python',
    ],
    include_package_data=True,
    packages=find_packages(),
    zip_safe=False,
)



import unittest

from django.conf import settings
settings.configure()

from django.contrib.sites.models import Site
from django.core.management.base import CommandError
from django.test.utils import override_settings
from unittest import TestCase

from disqus.api import DisqusClient
from disqus.templatetags.disqus_tags import disqus_dev


class FakeRequest(object):
    def __init__(self, path):
        self.path = path


# mock Site
class FakeSiteManager(object):
    def __init__(self, domain, name):
        self.site = Site(domain=domain, name=name)
        
    def get_current(self):
        return self.site


class DisqusTest(TestCase):

    def setUp(self):
        self.real_sites_manager = Site.objects

    def tearDown(self):
        Site.objects = self.real_sites_manager

    def test_client_init(self):
        """
        First, we test if the DisqusClient class can be initialized
        and parameters that were passed are set correctly.
        """

        c = DisqusClient(foo='bar', bar='foo')
        self.assertEqual('bar', c.foo)
        self.assertEqual('foo', c.bar)
        with self.assertRaises(AttributeError):
            c.baz

    def test_disqus_dev_sets_full_url(self):
        test_domain = 'example.org'
        url_path = '/path/to/page'
        full_url = 'http://%s%s' % (test_domain, url_path)
        context = {'request': FakeRequest(path=url_path)}
        # mock out Site manager
        Site.objects = FakeSiteManager(test_domain, 'test')
        with override_settings(DEBUG=True):
            generated_html = disqus_dev(context)
        self.assertIn(full_url, generated_html)

if __name__ == '__main__':
    unittest.main()



import datetime

from django import template
from django.conf import settings
from django.contrib.contenttypes.models import ContentType
from django.contrib.sites.models import Site
from django.contrib.syndication.views import Feed, add_domain
from django.utils import feedgenerator, tzinfo
from django.utils.encoding import force_unicode, iri_to_uri

USE_SINGLE_SIGNON = getattr(settings, "DISQUS_USE_SINGLE_SIGNON", False)

class WxrFeedType(feedgenerator.Rss201rev2Feed):
    def rss_attributes(self):
        return {
            u"version": self._version,
            u'xmlns:content': u"http://purl.org/rss/1.0/modules/content/",
            u'xmlns:dsq': u"http://www.disqus.com/",
            u'xmlns:dc': u"http://purl.org/dc/elements/1.1/",
            u'xmlns:wp': u"http://wordpress.org/export/1.0/",
        }
    
    def format_date(self, date):
        return date.strftime('%Y-%m-%d %H:%M:%S')
    
    def add_item(self, title, link, description, author_email=None,
        author_name=None, author_link=None, pubdate=None, comments=None,
        unique_id=None, enclosure=None, categories=(), item_copyright=None,
        ttl=None, **kwargs):
        """
        Adds an item to the feed. All args are expected to be Python Unicode
        objects except pubdate, which is a datetime.datetime object, and
        enclosure, which is an instance of the Enclosure class.
        """
        to_unicode = lambda s: force_unicode(s, strings_only=True)
        if categories:
            categories = [to_unicode(c) for c in categories]
        if ttl is not None:
            # Force ints to unicode
            ttl = force_unicode(ttl)
        item = {
            'title': to_unicode(title),
            'link': iri_to_uri(link),
            'description': to_unicode(description),
            'author_email': to_unicode(author_email),
            'author_name': to_unicode(author_name),
            'author_link': iri_to_uri(author_link),
            'pubdate': pubdate,
            'comments': comments,
            'unique_id': to_unicode(unique_id),
            'enclosure': enclosure,
            'categories': categories or (),
            'item_copyright': to_unicode(item_copyright),
            'ttl': ttl,
        }
        item.update(kwargs)
        self.items.append(item)
    
    def add_root_elements(self, handler):
        pass
    
    def add_item_elements(self, handler, item):
        if item['comments'] is None:
            return
        handler.addQuickElement(u"title", item['title'])
        handler.addQuickElement(u"link", item['link'])
        handler.addQuickElement(u"content:encoded", item['description'])
        handler.addQuickElement(u'dsq:thread_identifier', item['unique_id'])
        handler.addQuickElement(u'wp:post_date_gmt', 
            self.format_date(item['pubdate']).decode('utf-8'))
        handler.addQuickElement(u'wp:comment_status', item['comment_status'])
        self.write_comments(handler, item['comments'])
        
    def add_comment_elements(self, handler, comment):
        if USE_SINGLE_SIGNON:
            handler.startElement(u"dsq:remote", {})
            handler.addQuickElement(u"dsq:id", comment['user_id'])
            handler.addQuickElement(u"dsq:avatar", comment['avatar'])
            handler.endElement(u"dsq:remote")
        handler.addQuickElement(u"wp:comment_id", comment['id'])
        handler.addQuickElement(u"wp:comment_author", comment['user_name'])
        handler.addQuickElement(u"wp:comment_author_email", comment['user_email'])
        handler.addQuickElement(u"wp:comment_author_url", comment['user_url'])
        handler.addQuickElement(u"wp:comment_author_IP", comment['ip_address'])
        handler.addQuickElement(u"wp:comment_date_gmt", 
            self.format_date(comment['submit_date']).decode('utf-8'))
        handler.addQuickElement(u"wp:comment_content", comment['comment'])
        handler.addQuickElement(u"wp:comment_approved", comment['is_approved'])
        if comment['parent'] is not None:
            handler.addQuickElement(u"wp:comment_parent", comment['parent'])
    
    def write_comments(self, handler, comments):
        for comment in comments:
            handler.startElement(u"wp:comment", {})
            self.add_comment_elements(handler, comment)
            handler.endElement(u"wp:comment")


class BaseWxrFeed(Feed):
    feed_type = WxrFeedType
    
    def get_feed(self, obj, request):
        current_site = Site.objects.get_current()
        
        link = self._Feed__get_dynamic_attr('link', obj)
        link = add_domain(current_site.domain, link)
        feed = self.feed_type(
            title = self._Feed__get_dynamic_attr('title', obj),
            link = link,
            description = self._Feed__get_dynamic_attr('description', obj),
        )
        
        title_tmp = None
        if self.title_template is not None:
            try:
                title_tmp = template.loader.get_template(self.title_template)
            except template.TemplateDoesNotExist:
                pass
        
        description_tmp = None
        if self.description_template is not None:
            try:
                description_tmp = template.loader.get_template(self.description_template)
            except template.TemplateDoesNotExist:
                pass
        
        for item in self._Feed__get_dynamic_attr('items', obj):
            if title_tmp is not None:
                title = title_tmp.render(
                    template.RequestContext(request, {
                        'obj': item, 'site': current_site
                    }))
            else:
                title = self._Feed__get_dynamic_attr('item_title', item)
            if description_tmp is not None:
                description = description_tmp.render(
                    template.RequestContext(request, {
                        'obj': item, 'site': current_site
                    }))
            else:
                description = self._Feed__get_dynamic_attr('item_description', item)
            link = add_domain(
                current_site.domain,
                self._Feed__get_dynamic_attr('item_link', item),
            )
            
            pubdate = self._Feed__get_dynamic_attr('item_pubdate', item)
            if pubdate and not hasattr(pubdate, 'tzinfo'):
                ltz = tzinfo.LocalTimezone(pubdate)
                pubdate = pubdate.replace(tzinfo=ltz)
            
            feed.add_item(
                title = title,
                link = link,
                description = description,
                unique_id = self._Feed__get_dynamic_attr('item_guid', item, link),
                pubdate = pubdate,
                comment_status = self._Feed__get_dynamic_attr('item_comment_status', item, 'open'),
                comments = self._get_comments(item)
            )
        return feed
    
    def _get_comments(self, item):
        cmts = self._Feed__get_dynamic_attr('item_comments', item)
        output = []
        for comment in cmts:
            output.append({
                'user_id': self._Feed__get_dynamic_attr('comment_user_id', comment),
                'avatar': self._Feed__get_dynamic_attr('comment_avatar', comment),
                'id': str(self._Feed__get_dynamic_attr('comment_id', comment)),
                'user_name': self._Feed__get_dynamic_attr('comment_user_name', comment),
                'user_email': self._Feed__get_dynamic_attr('comment_user_email', comment),
                'user_url': self._Feed__get_dynamic_attr('comment_user_url', comment),
                'ip_address': self._Feed__get_dynamic_attr('comment_ip_address', comment),
                'submit_date': self._Feed__get_dynamic_attr('comment_submit_date', comment),
                'comment': self._Feed__get_dynamic_attr('comment_comment', comment),
                'is_approved': str(self._Feed__get_dynamic_attr('comment_is_approved', comment)),
                'parent': str(self._Feed__get_dynamic_attr('comment_parent', comment)),
            })
        return output
        

class ContribCommentsWxrFeed(BaseWxrFeed):
    link = "/"
    
    def item_comments(self, item):
        from django.contrib.comments.models import Comment
        
        ctype = ContentType.objects.get_for_model(item)
        return Comment.objects.filter(content_type=ctype, object_pk=item.pk)
    
    def item_guid(self, item):
        ctype = ContentType.objects.get_for_model(item)
        return "%s_%s" % (ctype.name, item.pk)
    
    def comment_id(self, comment):
        return comment.pk
    
    def comment_user_id(self, comment):
        return force_unicode(comment.user_id)
    
    def comment_user_name(self, comment):
        return force_unicode(comment.user_name)
    
    def comment_user_email(self, comment):
        return force_unicode(comment.user_email)
    
    def comment_user_url(self, comment):
        return force_unicode(comment.user_url)
    
    def comment_ip_address(self, comment):
        return force_unicode(comment.ip_address)
    
    def comment_submit_date(self, comment):
        return comment.submit_date
    
    def comment_comment(self, comment):
        return comment.comment
    
    def comment_is_approved(self, comment):
        return int(comment.is_public)
    
    comment_parent = 0
    